/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggraxl.AggrAXL
 *
 * de.inovat.dua.aggraxl.AggrAXL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggraxl.AggrAXL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggraxl.AggrAXL; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggraxl;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;

import java.util.*;

import static java.util.GregorianCalendar.MILLISECOND;
import static java.util.GregorianCalendar.MINUTE;
import static java.util.GregorianCalendar.SECOND;

//~ KLASSEN ===================================================================

/**
 * Klasse, die die Aggregation der Achslasdaten durchführt.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class AggrAXL implements StandardApplication, ClientReceiverInterface, ClientSenderInterface {

    /** WarteZeit in ms, bis die Auftragsliste erneut geprüft wird, nachdem keinerlei Aufträge mehr vorlagen * */
    private static final long WARTEZEIT_AUF_NAECHSTEN_AUFTRAG = 60 * 1000;  // 1 Minute
    private static int        _versionAxlEinzeldaten;

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug;

    //~ FELDER ================================================================

    private long _bisZeitPunkt;
    private long _bisZeitPunktAufrufParameter;

    /** Datenverteilerverbindung */
    private ClientDavInterface _dav;
    private boolean            _einmaligeNachBerechnung;
    private boolean            _einmaligeNachBerechnungAufrufParameter;
    private String             _grenzenDoppelAchsenAufrufParameter;
    private String             _grenzenDreifachAchsenAufrufParameter;
    private String             _grenzenEinzelAchsenAufrufParameter;
    private String             _grenzenFzKlasse3AufrufParameter;
    private String             _grenzenFzKlasse5AufrufParameter;
    private String             _grenzenFzKlasse8AufrufParameter;
    private String             _grenzenFzKlasse9AufrufParameter;
    private int                _intervallDauerInStunden;
    private String             _intervallDauerInStundenAufrufParameter;

    /** Die Konfiguration des DaV. */
    private DataModel           _konfiguration;
    private String              _listePidsAufrufParameter;
    private int                 _minimaleWarteZeitInMinuten;
    private String              _minimaleWarteZeitInMinutenAufrufParameter;
    private int                 _nachberechnungInStunden;
    private String              _nachberechnungInStundenAufrufParameter;
    private String              _pidVerwaltungsObjektAggrAxlAufrufParameter;
    private String              _versionAxlEinzeldatenAufrufParameter;
    private SystemObject        _verwaltungsObjektAggrAxl;
    private long                _vonZeitPunkt;
    private long                _vonZeitPunktAufrufParameter;
    private List<String>        _listePidsAxlDe               = new ArrayList<String>();
    private List<SystemObject>  _listeAxlDeSystemObjekte      = new ArrayList<SystemObject>();
    private List<String>        _listePidsAxlDeNichtVorhanden = new ArrayList<String>();
    private Map<String, VewAXL> _mapVewAXL                    = new HashMap<String, VewAXL>();

    //~ GET METHODEN ==========================================================

    public static int getVersionAxlEinzeldaten() {
        return _versionAxlEinzeldaten;
    }

    //~ METHODEN ==============================================================

    /**
     * Main-Methode der SWE Aggregation Achslastdaten.
     *
     * @param args Aufrufargumente der Applikation.
     */
    public static void main(String[] args) {
        AggrAXL aggrAXL = new AggrAXL();

        try {
            StandardApplicationRunner.run(aggrAXL, args);

            // Debug erst hier initialisieren, da erst ab hier verfügbar !!
            debug = Debug.getLogger();
            aggrAXL.auswertungAufrufParameterPidVerwaltungsObjekt();
            aggrAXL.auswertungAufrufParameterListePids();
            aggrAXL.auswertungAufrufParameterVonBisIntervallNachberechnung();
            aggrAXL.auswertungAufrufParameterVersionAxlEinzeldaten();
            aggrAXL.auswertungAufrufParameterGrenzenXXXParameter();
            aggrAXL.initialisierung();
            debug.config(aggrAXL.ausgabeKonfigurationsZusammenfassung());
        }
        catch (RuntimeException e) {
            e.printStackTrace();
            System.exit(-1);
        }

        aggrAXL.run();
    }

    /**
     * Gibt Informationen zur Konfiguration aus. Ist nur zu Anlaysezwecken bzw. zur zusammenfassenden Ausgabe der
     * Konfigurationsphase dieser Applikatione interessant.
     *
     * @return Informationen zur Konfiguration (Aufbau der Ausgabe kann sich ändern).
     */
    private String ausgabeKonfigurationsZusammenfassung() {
        StringBuffer sb = new StringBuffer();

        // Listen der SystemObjekte sortieren...
        Collections.sort(_listeAxlDeSystemObjekte, new Comparator<SystemObject>() {
            public int compare(SystemObject o1, SystemObject o2) {
                return o1.getPid().compareTo(o2.getPid());
            }
        });
        Collections.sort(_listePidsAxlDeNichtVorhanden);
        sb.append(String.format("%n%n"));
        sb.append(String.format("%n####################################################################################%n"));
        sb.append(String.format("START: Konfigurationszusammenfassung für AggrAXL"));
        sb.append(String.format("%n####################################################################################%n"));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -pidVerwaltungsObjektAggrAxl=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _pidVerwaltungsObjektAggrAxlAufrufParameter));

        if (_verwaltungsObjektAggrAxl != null) {
            sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _verwaltungsObjektAggrAxl.getPid()));
        } else {
            sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _verwaltungsObjektAggrAxl));
            sb.append(String.format("Verwaltungsobjekt wird nicht weiter verwendet.%nExterne Steuerung zur Laufzeit ist damit nicht mehr aktiviert.%n"));
        }

        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Im Aufrufparameter angegebene PIDs, zu denen keine SystemObjekte existieren"));
        sb.append(String.format("%n====================================================================================%n"));

        int i = 0;

        for (String pid : _listePidsAxlDeNichtVorhanden) {
            sb.append(String.format("%04d: %s%n", ++i, pid));
        }

        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Liste der PIDs der verwendeten Systemobjekte"));
        sb.append(String.format("%n====================================================================================%n"));

        int j = 0;

        for (SystemObject so : _listeAxlDeSystemObjekte) {
            sb.append(String.format("%04d: %s%n", ++j, so.getPidOrNameOrId()));
        }

        SimpleDateFormat datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -versionAxlEinzeldaten=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _versionAxlEinzeldatenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _versionAxlEinzeldaten));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -intervallDauerInStunden=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _intervallDauerInStundenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _intervallDauerInStunden));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -nachberechnungInStunden=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _nachberechnungInStundenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _nachberechnungInStunden));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -vonZeitPunkt=..."));
        sb.append(String.format("%n====================================================================================%n"));

        Date datum = new Date(_vonZeitPunktAufrufParameter);

        datum.toString();
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", datumsFormatierer.format(new Date(_vonZeitPunktAufrufParameter))));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", datumsFormatierer.format(new Date(_vonZeitPunkt))));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -bisZeitPunkt=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", datumsFormatierer.format(new Date(_bisZeitPunktAufrufParameter))));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", datumsFormatierer.format(new Date(_bisZeitPunkt))));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -einmaligeNachBerechnungAufrufParameter=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _einmaligeNachBerechnungAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _einmaligeNachBerechnung));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -minimaleWarteZeitInMinuten=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _minimaleWarteZeitInMinutenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", _minimaleWarteZeitInMinuten));

        Parameter p = Parameter.getInstanz();

        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenEinzelAchsen=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenEinzelAchsenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenEinzelAchsen())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenDoppelAchsen=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenDoppelAchsenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenDoppelAchsen())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenDreifachAchsen=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenDreifachAchsenAufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenDreifachAchsen())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenFzKlasse3=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenFzKlasse3AufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenFzKlasse3())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenFzKlasse5=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenFzKlasse5AufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenFzKlasse5())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenFzKlasse8=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenFzKlasse8AufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenFzKlasse8())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Aufrufparameter -grenzenFzKlasse9=..."));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Parameter beim Aufruf (Vorgabe)       : [%s]%n", _grenzenFzKlasse9AufrufParameter));
        sb.append(String.format("Parameter nach Auswertung (Verwendung): [%s]%n", p.ausgabe(p.getGrenzenFzKlasse9())));
        sb.append(String.format("%n====================================================================================%n"));
        sb.append(String.format("Auftragsliste initial..."));
        sb.append(String.format("%n====================================================================================%n"));

        int k = 0;

        for (Auftrag auftrag : AuftragsListe.getInstanz().holeKopieAuftragsListe()) {
            sb.append(String.format("%04d: %s%n", ++k, auftrag.toString()));
        }

        sb.append(String.format("%n%n"));
        sb.append(String.format("%n####################################################################################%n"));
        sb.append(String.format("ENDE: Konfigurationszusammenfassung für AggrAXL"));
        sb.append(String.format("%n####################################################################################%n"));

        return sb.toString();
    }

    /** Werte die Aufrufparameter final aus und setzt die davon abhängigen Parameter. */
    private void auswertungAufrufParameterGrenzenXXXParameter() {
        Parameter p = Parameter.getInstanz();

        p.initialisiereGrenzenEinzelAchsen(_grenzenEinzelAchsenAufrufParameter);
        p.initialisiereGrenzenDoppelAchsen(_grenzenDoppelAchsenAufrufParameter);
        p.initialisiereGrenzenDreifachAchsen(_grenzenDreifachAchsenAufrufParameter);
        p.initialisiereGrenzenFzKlasse3(_grenzenFzKlasse3AufrufParameter);
        p.initialisiereGrenzenFzKlasse5(_grenzenFzKlasse5AufrufParameter);
        p.initialisiereGrenzenFzKlasse8(_grenzenFzKlasse8AufrufParameter);
        p.initialisiereGrenzenFzKlasse9(_grenzenFzKlasse9AufrufParameter);
    }

    /**
     * Wertet die Aufrufparameter
     * <p/>
     * -listePids
     * <p/>
     * wie folgt aus:
     * <p/>
     * Es werden alle Systemobjekte vom Typ <tt>typ.deAxl</tt> ermittelt und dabei diejenigen für die weiteren
     * Berechnungen übernommen, deren Pid in der Liste der Pids aufgeführt ist (bzw. alle, falls Aufruf mit
     * <tt>-listePids=alle</tt> erfolgt) UND wenn die DE-Nummer != 255 ist. Wird kein passendes Objekt gefunden, wird
     * das Programm mit einer Fehlermeldung abgebrochen.
     */
    private void auswertungAufrufParameterListePids() {
        try {

            // Pids der zu berücksichtigenden Objekte ermitteln
            _listePidsAxlDe = Arrays.asList(_listePidsAufrufParameter.split("(:|,|;)"));

            // Alle möglichen SystemObjekte ermittlen...
            List<SystemObject> listeAlleAxlDeSystemObjekte = _konfiguration.getType("typ.deAxl").getElements();

            // ... und jetzt Liste mit zu verwendenden aufbauen (falls nicht "alle" verwendet werden sollen) ...
            // wobei vorher noch alle DE = 255 entfernt werden, da für diese keine Aggregation durchgeführt werden darf...
            boolean        isAlle = ((_listePidsAxlDe.size() == 1) && _listePidsAxlDe.get(0).equals("alle"));
            AttributeGroup atg    = _konfiguration.getAttributeGroup("atg.de");

            for (SystemObject so : listeAlleAxlDeSystemObjekte) {
                if ((isAlle || _listePidsAxlDe.contains(so.getPid())) && (so.getConfigurationData(atg).getUnscaledValue("DEKanal").intValue() != 255)) {
                    _listeAxlDeSystemObjekte.add(so);
                }
            }

            // .. und für Konfigurationszusammenfassung Liste der nicht verwendeten PID's aus dem Aufrufparameter ermitteln
            List<String> listePidsVerwendet = new ArrayList<String>();

            for (SystemObject so : _listeAxlDeSystemObjekte) {
                listePidsVerwendet.add(so.getPid());
            }

            for (String pid : _listePidsAxlDe) {
                if (!listePidsVerwendet.contains(pid)) {
                    _listePidsAxlDeNichtVorhanden.add(pid);
                }
            }
        }
        catch (Exception e) {
            debug.error("Es konnten keine Objekte vom typ.deAxl ermittelt werden!. Programm wird beendet.", e);

            throw new RuntimeException();
        }

        if (_listeAxlDeSystemObjekte.size() == 0) {
            debug.error("Es konnten keine Objekte vom typ.deAxl ermittelt werden!. Programm wird beendet.");

            throw new RuntimeException();
        }
    }

    /**
     * Wertet den Aufrufparameter
     * <p/>
     * -pidVerwaltungsObjektAggrAxl
     * <p/>
     * wie folgt aus:
     * <p/>
     * Ist der Wert <code>automatisch</code>, so wird das Objekt mit dem Typ <code>typ.aggrAxl</code> und der Pid
     * <code>aggrAxl.vewStandard1</code> verwendet, sonst das Objekt mit der im Aufrufparameter übergebenen Pid..
     */
    private void auswertungAufrufParameterPidVerwaltungsObjekt() {
        try {
            if (_pidVerwaltungsObjektAggrAxlAufrufParameter.equals("automatisch")) {
                _verwaltungsObjektAggrAxl = _konfiguration.getObject("aggrAxl.vewStandard1");
            } else {
                _verwaltungsObjektAggrAxl = _konfiguration.getObject(_pidVerwaltungsObjektAggrAxlAufrufParameter);
            }

            if ((_verwaltungsObjektAggrAxl != null) &&!_verwaltungsObjektAggrAxl.isOfType("typ.aggrAxl")) {
                debug.warning(String.format("VerwaltungsObjekt für Aggregation Achslast [-pidVerwaltungsObjektAggrAxl=%s] ist nicht vom Typ [typ.aggrAxl] sondern vom Typ [%s].%nVerwaltungsobjekt wird nicht weiter verwendet. Externe Steuerung zur Laufzeit ist damit nicht mehr aktiviert.", _pidVerwaltungsObjektAggrAxlAufrufParameter, _verwaltungsObjektAggrAxl.getType().getPid()));
            } else if (_verwaltungsObjektAggrAxl == null) {
                debug.warning(String.format("VerwaltungsObjekt für Aggregation Achslast [-pidVerwaltungsObjektAggrAxl=%s] konnte nicht ermittelt werden.%nVerwaltungsobjekt wird nicht weiter verwendet. Externe Steuerung zur Laufzeit ist damit nicht mehr aktiviert.", _pidVerwaltungsObjektAggrAxlAufrufParameter));
            } else {
                debug.info(String.format("VerwaltungsObjekt für Aggregation Achslast [-pidVerwaltungsObjektAggrAxl=%s] wird verwendet. Externe Steuerung zur Laufzeit ist aktiviert.", _pidVerwaltungsObjektAggrAxlAufrufParameter));
            }
        }
        catch (Exception e) {
            debug.warning(String.format("VerwaltungsObjekt für Aggregation Achslast [-pidVerwaltungsObjektAggrAxl=%s] konnte nicht ermittelt werden.%nVerwaltungsobjekt wird nicht weiter verwendet. Externe Steuerung zur Laufzeit ist damit nicht mehr aktiviert.", _pidVerwaltungsObjektAggrAxlAufrufParameter), e);
        }
    }

    private void auswertungAufrufParameterVersionAxlEinzeldaten() {

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -versionAxlEinzeldaten
        // ------------------------------------------------------------------------------------------------------
        try {
            _versionAxlEinzeldaten = new Integer(_versionAxlEinzeldatenAufrufParameter);

            if ((_versionAxlEinzeldaten < 0) || (_versionAxlEinzeldaten > 3)) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            _versionAxlEinzeldaten = 3;
            debug.warning(String.format("Aufrufparameter [-versionAxlEinzeldaten=%s] ist ungültig. Wert wurde für weiter Berechnung auf [-versionAxlEinzeldaten=%d] gesetzt.", _versionAxlEinzeldatenAufrufParameter, _versionAxlEinzeldaten));
        }
    }

    /**
     * Wertet die Aufrufparameter
     * <p/>
     * -vonZeitPunkt
     * <p/>
     * -bisZeitPunkt
     * <p/>
     * -intervallDauerInStunden
     * <p/>
     * -nachberechnungInStunden
     * <p/>
     * -einmaligeNachBerechnung
     * <p/>
     * wie folgt aus:
     * <p/>
     * <tt>-intervallDauerInStunden</tt>: Wird auf gültige Werte (1,2,3,4,6,8,12,24,48,72,96,120) geprüft und bei
     * ungültigem Wert auf 1 Stunde gesetzt.
     * <p/>
     * <tt>-nachberechnungInStunden</tt>: Wird auf Gültigkeit getestet (ganze Zahl) und bei ungültigem Wert auf 0
     * Stunden (keine Nachberechnung) gesetzt.
     * <p/>
     * <tt>-vonZeitPunkt</tt>: Wird auf gültigen Intervallanfangszeitpunkt abhängig vom Parameter
     * <tt>-intervallDauerInStunden</tt> und <tt>-nachberechnungInStunden</tt> auf denn nächsten, davorliegenden
     * Intervallbeginn gesetzt.
     * <p/>
     * Beispiel: Ist <tt>-intervallDauerInStunden=3</tt> und <tt>-vonZeitPunkt=04.11.2010 11:51:01,956</tt> und
     * <tt>-nachberechnungInStunden=7</tt>, so wird der Zeitpunkt auf <tt>04.11.2010 03:00:00,000</tt> gesetzt.
     * <p/>
     * <tt>-bisZeitPunkt</tt>: Wird auf gültigen Intervallanfangszeitpunkt abhängig vom Parameter
     * <tt>-intervallDauerInStunden</tt> auf denn nächsten, dahinterliegenden Intervallbeginn gesetzt. Beispiel: Ist
     * <tt>-intervallDauerInStunden=3</tt> und <tt>-bisZeitPunkt=04.11.2010 11:51:01,956</tt>, so wird der Zeitpunkt auf
     * <tt>04.11.2010 12:00:00,000</tt> gesetzt. Liegt der so ermittelte bisZeitPunkt nicht mindestens ein Intervall
     * hinter dem vonZeitPunkt, wird er automatisch dahingehend korrigiert. Der bisZeitPunkt wird nur berücksichtigt,
     * wenn der Parameter <tt>-einmaligeNachBerechnung=ja</tt> ist. Ist der Parameter
     * <tt>-einmaligeNachBerechnung=nein</tt>, so wird ab dem Startzeitpunkt bis zum Beenden des Prozesses die
     * Aggregation durchgeführt.
     */
    private void auswertungAufrufParameterVonBisIntervallNachberechnung() {

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -nachberechnungInStunden
        // ------------------------------------------------------------------------------------------------------
        try {
            _nachberechnungInStunden = new Integer(_nachberechnungInStundenAufrufParameter);

            if (_nachberechnungInStunden < 0) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            _nachberechnungInStunden = 0;
            debug.warning(String.format("Aufrufparameter [-nachberechnungInStunden=%s] ist ungültig. Wert wurde für weiter Berechnung auf [-nachberechnungInStunden=%d] gesetzt.", _nachberechnungInStundenAufrufParameter, _nachberechnungInStunden));
        }

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -intervallDauerInStunden
        // ------------------------------------------------------------------------------------------------------
        try {
            Set<Integer> gueltigeWerte = new HashSet<Integer>();

            gueltigeWerte.add(1);
            gueltigeWerte.add(2);
            gueltigeWerte.add(3);
            gueltigeWerte.add(4);
            gueltigeWerte.add(6);
            gueltigeWerte.add(8);
            gueltigeWerte.add(12);
            gueltigeWerte.add(24);
            gueltigeWerte.add(48);
            gueltigeWerte.add(72);
            gueltigeWerte.add(96);
            gueltigeWerte.add(120);
            _intervallDauerInStunden = new Integer(_intervallDauerInStundenAufrufParameter);

            if (!gueltigeWerte.contains(_intervallDauerInStunden)) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            _intervallDauerInStunden = 1;
            debug.warning(String.format("Aufrufparameter [-intervallDauerInStunden=%s] ist ungültig (nur 1,2,3,4,6,8,12,24,48,72,96,120 zulässig). Wert wurde für weiter Berechnung auf [-intervallDauerInStunden=%d] gesetzt.", _intervallDauerInStundenAufrufParameter, _intervallDauerInStunden));
        }

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -vonZeitPunkt
        // ------------------------------------------------------------------------------------------------------
        _vonZeitPunkt = ermittleIntervallBeginn(_vonZeitPunktAufrufParameter, _intervallDauerInStunden, _nachberechnungInStunden);

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -bisZeitPunkt
        // ------------------------------------------------------------------------------------------------------
        _bisZeitPunkt = ermittleIntervallEnde(_bisZeitPunktAufrufParameter, _intervallDauerInStunden);

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -einmaligeNachBerechnung
        // ------------------------------------------------------------------------------------------------------
        _einmaligeNachBerechnung = _einmaligeNachBerechnungAufrufParameter;

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -minimaleWarteZeitInMinuten
        // ------------------------------------------------------------------------------------------------------
        try {
            _minimaleWarteZeitInMinuten = new Integer(_minimaleWarteZeitInMinutenAufrufParameter);

            if (_minimaleWarteZeitInMinuten < 0) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            _minimaleWarteZeitInMinuten = 5;
            debug.warning(String.format("Aufrufparameter [-minimaleWarteZeitInMinuten=%s] ist ungültig. Wert wurde für weiter Berechnung auf [-minimaleWarteZeitInMinuten=%d] gesetzt.", _minimaleWarteZeitInMinutenAufrufParameter, _minimaleWarteZeitInMinuten));
        }
    }

    /**
     * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
     * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer Quelle
     * oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr für die zuvor angemeldeten Daten vorhanden
     * ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung
     *                        bezieht.
     * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
     *                        bezieht.
     * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
     *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
     *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
     *
     * @see #START_SENDING
     * @see #STOP_SENDING
     * @see #STOP_SENDING_NO_RIGHTS
     * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
     */
    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

        // tut nicht, da Sendesteuerung nicht unterstützt wird
    }

    /**
     * Ermittelt den IntervallBeginn ausgehend von einem Zeitpunk, der Intervalldauer und der Nachberechnungszeit
     *
     * @param spaetesterIntervallBeginn Anfangszeitpunkt in ms seit 1.1.1970, ab dem spätestens die Daten berechnet
     *                                  werden sollen. Dieser Wert wird dann noch unter Berücksichtigung der Parameter
     *                                  <tt>intervallDauerInStunden</tt> und <tt>nachberechnungInStunden</tt> soweit in
     *                                  die Vergangenheit verschoben, dass genau ein gültiger Intervallbeginn ermittelt
     *                                  wird.
     * @param intervallDauerInStunden   Intervalldauer, die gerechnet werden soll.
     * @param nachberechnungInStunden   Ggf. Zeit die vor dem <tt>vonZeitPunkt</tt> zusätzlich noch berücksichtigt
     *                                  werden soll.
     *
     * @return Intervallbeginn (GENAU!!), der vor dem Zeitpunkt <tt>vonZeitPunkt</tt>
     */
    private long ermittleIntervallBeginn(long spaetesterIntervallBeginn, int intervallDauerInStunden, int nachberechnungInStunden) {
        Calendar vonZeitPunkt = new GregorianCalendar();

        // vonZeitPunkt darf spätestens sein...
        vonZeitPunkt.setTimeInMillis(spaetesterIntervallBeginn - (nachberechnungInStunden * 60 * 60 * 1000));

        // ... und muss jetzt auf gültigen Intervallzeitpunkt DAVOR abgebildet werden...
        // ToDo: Vereinfachte Version, liefert nur für Intervalldauer = 1h korrekte Werte, indem einfach die Minuten,
        switch (intervallDauerInStunden) {
        case 1 :

            // Sekunden und Millisekunden auf 0 gesetzt werden.
            vonZeitPunkt.set(MILLISECOND, 0);
            vonZeitPunkt.set(SECOND, 0);
            vonZeitPunkt.set(MINUTE, 0);

            break;

        default :
            debug.warning("IntervallDauer != 1 Stunde wird aktuell nicht unterstützt! Intervallbeginn wird für Intervalldauer == 1 Stunde berechnet.");

            // Sekunden und Millisekunden auf 0 gesetzt werden.
            vonZeitPunkt.set(MILLISECOND, 0);
            vonZeitPunkt.set(SECOND, 0);
            vonZeitPunkt.set(MINUTE, 0);
        }

        return vonZeitPunkt.getTimeInMillis();
    }

    /**
     * Ermittelt das IntervallEnde ausgehend von einem Zeitpunkt und der Intervalldauer
     *
     * @param fruehestesIntervallEnde Endzeitpunkt, bis zu dem mindestens gerechnet werden muss.
     * @param intervallDauerInStunden Intervalldauer, die gerechnet werden soll.
     *
     * @return Intervallende (AUSSCHLIESSLICH!!), also der Intervallbeginn des nächsten Intervalls
     */
    private long ermittleIntervallEnde(long fruehestesIntervallEnde, int intervallDauerInStunden) {
        return ermittleIntervallBeginn(fruehestesIntervallEnde + (intervallDauerInStunden * 60 * 60 * 1000), intervallDauerInStunden, 0);
    }

    /**
     * Folgeauftrag für den aktuell übergebenen Auftrag berechnen.
     * <p/>
     * Der Folgeauftrag liegt genau eine Intervalllänge später, sofern dieser Zeitpunkt nach innerhalb des insgesamt zu
     * berücksichtigenden Zeitbereichs liegt (Aufrufparameter <tt>-vonZeitPunkt, -bisZeitPunkt,
     * -einmaligeNachBerechnung</tt>.
     *
     * @param auftrag Der aktuelle Auftrag, für den ein Folgeauftrag berechnet werden soll.
     *
     * @return Folgeauftrag für den aktuell als Parameter übergebenen Auftrag oder <code>null</code>, wenn kein
     *         Folgeauftrag mehr existiert.
     */
    private Auftrag erstelleFolgeAuftrag(Auftrag auftrag) {

        // Prüfen, ob Folgeauftrag überhaupt noch berechnet werden soll, weil Ursprungsauftrag Einmalauftrag war:
        if (auftrag.isEinmalAuftrag()) {
            return null;
        }

        // Sonst Folgeauftrag zusammenstellen und prüfen, ob dieser überhaupt noch ausgeführt werden soll.
        Auftrag folgeAuftrag = new Auftrag(auftrag);

        // Prüfen, ob Folgeauftrag überhaupt noch berechnet werden soll, weil Gesamtzeitraum nur einmalig berechnet werden soll:
        if (_einmaligeNachBerechnung && (folgeAuftrag.getIntervallEnde() > _bisZeitPunkt)) {
            return null;
        }

        return folgeAuftrag;
    }

    /** Initialisierung von AggrAxl */
    private void initialisierung() {

        // VewAXL-Objekte konstruieren
        for (SystemObject axlDe : _listeAxlDeSystemObjekte) {
            VewAXL vewAXL = new VewAXL(_dav, axlDe);

            _mapVewAXL.put(axlDe.getPid(), vewAXL);
        }

        // Initiales setzen der Auftragsliste auf Basis der VewAXL und der Aurufparameter
        for (VewAXL vewAXL : _mapVewAXL.values()) {
            long          intervallBeginn       = _vonZeitPunkt;
            long          intervallEnde         = intervallBeginn + (_intervallDauerInStunden * 60 * 60 * 1000);
            long          ausfuehrungsZeitpunkt = intervallEnde + (_minimaleWarteZeitInMinuten * 60 * 1000);
            Auftrag       auftrag               = new Auftrag(ausfuehrungsZeitpunkt, intervallBeginn, intervallEnde, vewAXL, false);
            AuftragsListe auftragsListe         = AuftragsListe.getInstanz();

            auftragsListe.addAuftrag(auftrag);
        }

        // Anmeldung des Verwaltungsobjekts und der ATG zum Empfangen
        DataDescription datenBeschreibung;

        // Anmeldung auf Empfang von externen Vorgaben über DAV
        datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.aggrAxlSteuerung"), _konfiguration.getAspect("asp.anfrage"));
        _dav.subscribeReceiver(this, _verwaltungsObjektAggrAxl, datenBeschreibung, ReceiveOptions.normal(), ReceiverRole.drain());
    }

    /**
     * Initialisierung. Wird durch den {@link de.bsvrz.sys.funclib.application.StandardApplicationRunner} aufgerufen.
     *
     * @param dav Die Datenverteilerverbindung.
     */
    @Override
    public void initialize(ClientDavInterface dav) {
        _dav           = dav;
        _konfiguration = _dav.getDataModel();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen erwünscht
     * sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser Methode
     * dürfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B. Konfigurationsanfragen)
     * durchgeführt werden, da ansonsten ein Deadlock entsteht.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene System-Objekt.
     * @param dataDescription Die in der zugehörigen Sendeanmeldung angegebenen beschreibenden Informationen der
     *                        angemeldeten Daten.
     *
     * @return <code>true</code>, falls Sendesteuerungen gewünscht sind, sonst <code>false</code>.
     *
     * @see #dataRequest
     */
    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }

    //~ METHODEN ==============================================================

    /**
     * Wertet die Aufrufparameter aus.
     *
     * @param argumentList Aufrufparamter der Applikation
     *
     * @throws Exception Wenn die Aufrufparameter nicht vollständig ausgewertet werden können.
     */
    @Override
    public void parseArguments(ArgumentList argumentList) throws Exception {
        debug = Debug.getLogger();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -pidVerwaltungsObjekt
        // ------------------------------------------------------------------------------------------------------
        _pidVerwaltungsObjektAggrAxlAufrufParameter = argumentList.fetchArgument("-pidVerwaltungsObjektAggrAxl=automatisch").asNonEmptyString();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -listePids
        // ------------------------------------------------------------------------------------------------------
        _listePidsAufrufParameter = argumentList.fetchArgument("-listePids=deAxl.002.eak.002.sm.10521-063.uz.dauz.axlh:deAxl.001.eak.002.sm.10521-063.uz.dauz.axl").asNonEmptyString();

        // _listePidsAufrufParameter = argumentList.fetchArgument("-listePids=deAxl.002.eak.002.sm.10521-063.uz.dauz.axl").asNonEmptyString();
        // _listePidsAufrufParameter = argumentList.fetchArgument("-listePids=deAxl.002.eak.002.sm.10521-063.uz.dauz.axl.xx").asNonEmptyString();
        // _listePidsAufrufParameter = argumentList.fetchArgument("-listePids=alle").asNonEmptyString();
        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -vonZeitPunkt
        // ------------------------------------------------------------------------------------------------------
        try {
            _vonZeitPunktAufrufParameter = argumentList.fetchArgument("-vonZeitPunkt=jetzt").asTime();
            _vonZeitPunktAufrufParameter = argumentList.fetchArgument("-vonZeitPunkt=07:59:01,001").asTime();
            _vonZeitPunktAufrufParameter = argumentList.fetchArgument("-vonZeitPunkt=12:59:01,001").asTime();
        }
        catch (Exception e) {
            debug.error(String.format("Ungültiges Argument beim Aufrufparameter [-vonZeitPunkt]. Programm wird beendet.%nFehler: %s", e));

            throw new RuntimeException(e);
        }

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -bisZeitPunkt
        // ------------------------------------------------------------------------------------------------------
        try {
            _bisZeitPunktAufrufParameter = argumentList.fetchArgument("-bisZeitPunkt=jetzt").asTime();
        }
        catch (Exception e) {
            debug.error(String.format("Ungültiges Argument beim Aufrufparameter [-bisZeitPunkt]. Programm wird beendet.%nFehler: %s", e));

            throw new RuntimeException(e);
        }

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -intervallDauerInStunden
        // ------------------------------------------------------------------------------------------------------
        _intervallDauerInStundenAufrufParameter = argumentList.fetchArgument("-intervallDauerInStunden=1").asNonEmptyString().trim();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -versionAxlEinzeldaten
        // ------------------------------------------------------------------------------------------------------
        _versionAxlEinzeldatenAufrufParameter = argumentList.fetchArgument("-versionAxlEinzeldaten=3").asNonEmptyString().trim();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -nachberechnungInStunden
        // ------------------------------------------------------------------------------------------------------
        _nachberechnungInStundenAufrufParameter = argumentList.fetchArgument("-nachberechnungInStunden=5").asNonEmptyString().trim();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -einmaligeNachBerechnungAufrufParameter
        // ------------------------------------------------------------------------------------------------------
        try {
            _einmaligeNachBerechnungAufrufParameter = argumentList.fetchArgument("-einmaligeNachBerechnung=ja").booleanValue();
        }
        catch (Exception e) {
            debug.error(String.format("Ungültiges Argument beim Aufrufparameter [-einmaligeNachBerechnung]. Programm wird beendet.%nFehler: %s", e));

            throw new RuntimeException(e);
        }

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: -minimaleWarteZeitInMinuten
        // ------------------------------------------------------------------------------------------------------
        _minimaleWarteZeitInMinutenAufrufParameter = argumentList.fetchArgument("-minimaleWarteZeitInMinuten=5").asNonEmptyString().trim();

        // ------------------------------------------------------------------------------------------------------
        // Aufrufparameter: _grenzenXXX
        // ------------------------------------------------------------------------------------------------------
        _grenzenDoppelAchsenAufrufParameter   = argumentList.fetchArgument("-grenzenDoppelAchsen=4000:8000:12000:16000:18000:19000:20000:22000").asNonEmptyString().trim();
        _grenzenDreifachAchsenAufrufParameter = argumentList.fetchArgument("-grenzenDreifachAchsen=3000:6000:9000:12000:15000:18000:21000:24000:27000:30000").asNonEmptyString().trim();
        _grenzenEinzelAchsenAufrufParameter   = argumentList.fetchArgument("-grenzenEinzelAchsen=2000:4000:6000:8000:10000:11005:12000:13000:14000:15000").asNonEmptyString().trim();
        _grenzenFzKlasse3AufrufParameter      = argumentList.fetchArgument("-grenzenFzKlasse3=3500:7500:12000:14000:20000:26000:32000:38000").asNonEmptyString().trim();
        _grenzenFzKlasse5AufrufParameter      = argumentList.fetchArgument("-grenzenFzKlasse5=7500:14000:20000:26000:32000").asNonEmptyString().trim();
        _grenzenFzKlasse8AufrufParameter      = argumentList.fetchArgument("-grenzenFzKlasse8=7500:12000:14000:20000:26000:32000:38000:40000:44000:50000").asNonEmptyString().trim();
        _grenzenFzKlasse9AufrufParameter      = argumentList.fetchArgument("-grenzenFzKlasse9=7500:12000:14000:20000:26000:32000:38000:40000:44000:50000").asNonEmptyString().trim();
    }

    // Startet die eigentliche Aggregation, werte die Auftragsliste aus und aggregiert die Daten.
    private void run() {
        debug.info(String.format("%n=================================================================%n" + "Programm Aggregation Achslastdaten gestartet...%n" + "=================================================================%n"));

        // Auftragsliste holen
        AuftragsListe auftragsListe = AuftragsListe.getInstanz();

        // Anmeldung des Verwaltungsobjekts und der ATG zum Senden
        DataDescription datenBeschreibung;

        // Anmeldung auf das Versenden von Statusinformationen / Fortschrittsanzeige
        datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.aggrAxlStatus"), _konfiguration.getAspect("asp.aggrAxlStatus"));

        // Datensatz für die Statusinformationen
        Data statusInfo = _dav.createData(_konfiguration.getAttributeGroup("atg.aggrAxlStatus"));

        try {
            _dav.subscribeSender(this, _verwaltungsObjektAggrAxl, datenBeschreibung, SenderRole.source());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.error(String.format("Versuch der Doppelanmeldung der Statusanzeige mit Datenbeschreibung [%s]%nProgramm wird beendet.", datenBeschreibung), e);
            System.exit(-1);
        }

        // Endlosschleife mit Prüfung der Auftragsliste.
        // Sobald keinerlei Aufträge mehr vorhanden sind, wird das Programm beendet
        while (auftragsListe.hatNochAuftraege()) {
            Auftrag auftrag;
            Auftrag folgeAuftrag;

            if ((auftrag = auftragsListe.holeNaechstenAuftrag()) != null) {
                StringBuffer statusMeldung = new StringBuffer();

                // Ggf. Folgeauftrag in Liste eintragen
                folgeAuftrag = erstelleFolgeAuftrag(auftrag);
                statusMeldung.append(String.format("Aktueller Auftrag: %s", auftrag));

                if (folgeAuftrag != null) {
                    auftragsListe.addAuftrag(folgeAuftrag);
                    statusMeldung.append(String.format("%nFolgeauftrag     : %s", folgeAuftrag));
                } else {
                    statusMeldung.append(String.format("%nFolgeauftrag     : %s", "kein weiterer Folgeauftrag mehr zu berücksichtigen"));
                }

                debug.info(statusMeldung.toString());

                // Ausgabe Status 10% - Auftrag wird vorbereitet
                statusInfo.getUnscaledValue("FortschrittProzent").set(10);
                statusInfo.getTextValue("StatusMeldung").setText(statusMeldung.toString());

                try {
                    _dav.sendData(new ResultData(_verwaltungsObjektAggrAxl, datenBeschreibung, System.currentTimeMillis(), statusInfo));
                }
                catch (SendSubscriptionNotConfirmed e) {
                    debug.warning("Versuch des Versands von Statusinformationen ohne Sendefreigabe");
                }

                // Ausgabe Status 20% - Auftrag wird bearbeitet...
                statusMeldung.append(String.format("%nAggregations wird durchgeführt..."));
                statusInfo.getUnscaledValue("FortschrittProzent").set(20);
                statusInfo.getTextValue("StatusMeldung").setText(statusMeldung.toString());

                try {
                    _dav.sendData(new ResultData(_verwaltungsObjektAggrAxl, datenBeschreibung, System.currentTimeMillis(), statusInfo));
                }
                catch (SendSubscriptionNotConfirmed e) {
                    debug.warning("Versuch des Versands von Statusinformationen ohne Sendefreigabe");
                }

                // Ausgabe Status 100% - Auftrag wurde bearbeitet
                statusMeldung.append(String.format("%nAuftrag durchgeführt:%n%s", auftrag.ausfuehren()));
                statusInfo.getUnscaledValue("FortschrittProzent").set(100);
                statusInfo.getTextValue("StatusMeldung").setText(statusMeldung.toString());

                try {
                    _dav.sendData(new ResultData(_verwaltungsObjektAggrAxl, datenBeschreibung, System.currentTimeMillis(), statusInfo));
                }
                catch (SendSubscriptionNotConfirmed e) {
                    debug.warning("Versuch des Versands von Statusinformationen ohne Sendefreigabe");
                }

                debug.info(String.format("Ausgeführter Auftrag: %s", auftrag));
            } else {

                // Zurzeit ist nichts zu tun, also warten und es dann noch mal versuchen...
                debug.info(String.format("Zur Zeit keine sofort auszuführenden Aufträge in der Auftragsliste. Warte %d Sekunden.", WARTEZEIT_AUF_NAECHSTEN_AUFTRAG / 1000));

                try {
                    Thread.sleep(WARTEZEIT_AUF_NAECHSTEN_AUFTRAG);
                }
                catch (InterruptedException e) {
                    debug.info("Sleep der Auftragsbearbeitung wurde vorzeitig unterbrochen: ", e);
                }
            }
        }

        debug.info(String.format("%n=================================================================%n" + "Programm Aggregation Achslastdaten wird ordnungsgemäß beendet,%n" + "da keinerlei Aufträge (weder aktuell noch zukünftig) anliegen%n" + "=================================================================%n"));
        System.exit(0);
    }

    /**
     * Aktualisierungsmethode, die nach Empfang eines angemeldeten Datensatzes von den
     * Datenverteiler-Applikationsfunktionen aufgerufen wird. Diese Methode muss von der Applikation zur Verarbeitung
     * der empfangenen Datensätze implementiert werden.
     *
     * @param results Feld mit den empfangenen Ergebnisdatensätzen.
     */
    @Override
    public void update(ResultData[] results) {
        AuftragsListe auftragsListe = AuftragsListe.getInstanz();

        // Ergebnisdatensatz mit zusätzlichen Einmalaufträgen auswerten und in die Auftragsliste eintragen
        for (ResultData ergebnis : results) {
            if (ergebnis.hasData()) {
                Data         daten      = ergebnis.getData();
                long         von        = daten.getTimeValue("von").getMillis();
                long         bis        = daten.getTimeValue("bis").getMillis();
                int          anz        = daten.getArray("DeAxl").getLength();
                List<String> listeDeAxl = new ArrayList<String>();

                if (anz == 0) {

                    // Für alle DeAxl die Aufträge eintragen
                    listeDeAxl = _listePidsAxlDe;
                } else {
                    for (int i = 0; i < anz; i++) {
                        listeDeAxl.add(daten.getArray("DeAxl").getItem(i).asReferenceValue().getSystemObjectPid());
                    }
                }

                // Intervallzeiträume ermitteln...
                // von und bis auf korrekten Intervallanfang und Intervallende setzten, Anzahl der Intervalle berechnen
                von = ermittleIntervallBeginn(von, _intervallDauerInStunden, 0);
                bis = ermittleIntervallEnde(bis, _intervallDauerInStunden);

                long anzIntervalle = (bis - von) / (_intervallDauerInStunden * 60 * 60 * 1000);

                // Aufträge eintragen (für alle DeAxl und alle Intervalle)
                for (String pid : listeDeAxl) {
                    VewAXL vewAXL = _mapVewAXL.get(pid);

                    if (vewAXL != null) {
                        for (int i = 0; i < anzIntervalle; i++) {
                            long    intervallVon = von + (i * _intervallDauerInStunden * 60 * 60 * 1000);
                            long    intervallBis = intervallVon + (_intervallDauerInStunden * 60 * 60 * 1000);
                            Auftrag auftrag      = new Auftrag(intervallBis + _minimaleWarteZeitInMinuten * 60 * 1000, intervallVon, intervallBis, vewAXL, true);

                            auftragsListe.addAuftrag(auftrag);
                            debug.info(String.format("Auftrag aus externer Vorgabe zur Auftragsliste hinzugefügt: %s%n", auftrag));
                        }
                    }
                }
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.09.14
