/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.aggraxl.Parameter
 *
 * de.inovat.dua.aggraxl.Parameter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.aggraxl.Parameter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.aggraxl.Parameter; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.aggraxl;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * Singletonklasse zur Verwaltung der Parameter, die für die Aggregation benötigt werden.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author: HCK $)
 */
public class Parameter {

    /** DebugLogger für Debug-Ausgaben. */
    private static final Debug debug             = Debug.getLogger();
    private static Parameter   dieEinzigeInstanz = new Parameter();

    //~ FELDER ================================================================

    private long[] _grenzenDoppelAchsen   = { 4000, 8000, 12000, 16000, 18000, 19000, 20000, 22000 };
    private long[] _grenzenDreifachAchsen = { 3000, 6000, 9000, 12000, 15000, 18000, 21000, 24000, 27000, 30000 };
    private long[] _grenzenEinzelAchsen   = { 2000, 4000, 6000, 8000, 10000, 11005, 12000, 13000, 14000, 15000 };
    private long[] _grenzenFzKlasse3      = { 3500, 7500, 12000, 14000, 20000, 26000, 32000, 38000 };
    private long[] _grenzenFzKlasse5      = { 7500, 14000, 20000, 26000, 32000 };
    private long[] _grenzenFzKlasse8      = { 7500, 12000, 14000, 20000, 26000, 32000, 38000, 40000, 44000, 50000 };
    private long[] _grenzenFzKlasse9      = { 7500, 12000, 14000, 20000, 26000, 32000, 38000, 40000, 44000, 50000 };

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    private Parameter() {

        // Hier die Konstruktion der Instanz durchführen
    }

    //~ GET METHODEN ==========================================================

    public static Parameter getInstanz() {
        return dieEinzigeInstanz;
    }

    //~ METHODEN ==============================================================

    public String ausgabe(long[] parameter) {
        StringBuffer sb = new StringBuffer();

	    for (long wert : parameter) {
		    sb.append(wert);
		    sb.append(":");
	    }

        // letzten : wieder entfernen
        sb.replace(sb.length() - 1, sb.length(), "");

        return sb.toString();
    }

    //~ GET METHODEN ==========================================================

    public long[] getGrenzenDoppelAchsen() {
        return _grenzenDoppelAchsen;
    }

    public long[] getGrenzenDreifachAchsen() {
        return _grenzenDreifachAchsen;
    }

    public long[] getGrenzenEinzelAchsen() {
        return _grenzenEinzelAchsen;
    }

    public long[] getGrenzenFzKlasse3() {
        return _grenzenFzKlasse3;
    }

    public long[] getGrenzenFzKlasse5() {
        return _grenzenFzKlasse5;
    }

    public long[] getGrenzenFzKlasse8() {
        return _grenzenFzKlasse8;
    }

    public long[] getGrenzenFzKlasse9() {
        return _grenzenFzKlasse9;
    }

    //~ METHODEN ==============================================================

    public void initialisiereGrenzenDoppelAchsen(String grenzenDoppelAchsenAufrufParameter) {
        String[] einzelParam = grenzenDoppelAchsenAufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenDoppelAchsen = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenDoppelAchsen=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenDoppelAchsenAufrufParameter));
        }
    }

    public void initialisiereGrenzenDreifachAchsen(String grenzenDreifachAchsenAufrufParameter) {
        String[] einzelParam = grenzenDreifachAchsenAufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenDreifachAchsen = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenDreifachAchsen=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenDreifachAchsenAufrufParameter));
        }
    }

    public void initialisiereGrenzenEinzelAchsen(String grenzenEinzelAchsenAufrufParameter) {
        String[] einzelParam = grenzenEinzelAchsenAufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenEinzelAchsen = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenEinzelAchsen=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenEinzelAchsenAufrufParameter));
        }
    }

    public void initialisiereGrenzenFzKlasse3(String grenzenFzKlasse3AufrufParameter) {
        String[] einzelParam = grenzenFzKlasse3AufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenFzKlasse3 = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenFzKlasse3=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenFzKlasse3AufrufParameter));
        }
    }

    public void initialisiereGrenzenFzKlasse5(String grenzenFzKlasse5AufrufParameter) {
        String[] einzelParam = grenzenFzKlasse5AufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenFzKlasse5 = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenFzKlasse5=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenFzKlasse5AufrufParameter));
        }
    }

    public void initialisiereGrenzenFzKlasse8(String grenzenFzKlasse8AufrufParameter) {
        String[] einzelParam = grenzenFzKlasse8AufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenFzKlasse8 = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenFzKlasse8=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenFzKlasse8AufrufParameter));
        }
    }

    public void initialisiereGrenzenFzKlasse9(String grenzenFzKlasse9AufrufParameter) {
        String[] einzelParam = grenzenFzKlasse9AufrufParameter.split("(,|;|:)");
        long[]   parameter   = new long[einzelParam.length];

        try {
            for (int i = 0; i < einzelParam.length; i++) {
                parameter[i] = new Long(einzelParam[i]);
            }

            _grenzenFzKlasse9 = parameter;
        }
        catch (NumberFormatException e) {
            debug.warning(String.format("Aufrufparameter [-grenzenFzKlasse9=%s] konnte nicht ausgewertet werden. Vorgaben werden auf BAStStandard gesetzt.", grenzenFzKlasse9AufrufParameter));
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.12.10
